package com.pgmriver.android;

import java.nio.FloatBuffer;

import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.opengl.GLU;

public class GLViewFifth extends BaseGLView {

	public GLViewFifth(Context context) {
		super(context);
		// TODO Auto-generated constructor stub
		
		mTriangleBuffer = makeFloatBuffer(triangle);
		mColorBuffer = makeFloatBuffer(colors);
	}
	
	@Override
	protected void init(GL10 gl)
	{
		gl.glClearColor(0, 0, 0, 0);
		gl.glMatrixMode(GL10.GL_PROJECTION);
		gl.glLoadIdentity();
		
		// 左下(0,0f, 0.0f),右上(0.0f, 1.0f)をクリッピング領域とする
		// 投影行列を作成し、カレントの投影行列に乗算する
		GLU.gluOrtho2D(gl, 0.0f, 1.3f, 0.0f, 1.0f);
		
		// デプステストをすると、
		// 隠れている部分はレンダリングされない(陰面消去)
		gl.glEnable(GL10.GL_DEPTH_TEST);
		gl.glDepthFunc(GL10.GL_LEQUAL);
		
		// デプスバッファをクリアする(各フレームを再描画する前に必ず、バッファをクリアする)
		gl.glClearDepthf(1.0f);
		
		// GL_FLAT : 単色(全頂点同じ色)
		// GL_SMOOTH : 複数の色(頂点間の色を補間する)
		gl.glShadeModel(GL10.GL_SMOOTH);
	}

	@Override
	protected void drawFrame(GL10 gl) {
		// TODO Auto-generated method stub
		
		gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);
		
		gl.glMatrixMode(GL10.GL_MODELVIEW);
		gl.glLoadIdentity();
		gl.glTranslatef(0, 0, -1);
		
		gl.glPushMatrix();
		gl.glVertexPointer(3, GL10.GL_FLOAT, 0, mTriangleBuffer);
		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		
		gl.glColorPointer(4, GL10.GL_FLOAT, 0, mColorBuffer);
		gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
		
		gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, 0, 3);
		gl.glPopMatrix();
		
		gl.glPushMatrix();
		gl.glTranslatef(-0.2f, -.1f, -1.0f);
		gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, 0, 3);
		gl.glPopMatrix();
		
		gl.glPushMatrix();
		gl.glTranslatef(0.1f, 0.1f, -0.5f);
		gl.glRotatef(45.0f, 0, 0, 1);
		gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, 0, 3);
		gl.glPopMatrix();
		
		gl.glTranslatef(0.15f, 0.05f, -0.4f);
		gl.glRotatef(45, 0, 1, 0);
		gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, 0, 3);
		gl.glPopMatrix();
	}

	float[] triangle = new float[] {
			-0.25f, -0.25f, 0.0f,
			0.25f, -0.25f, 0.0f,
			-0.25f, 0.25f, 0.0f};

	float[] colors = new float[] {
			1, 0, 0, 1,
			0, 1, 0, 1,
			0, 0, 1, 1};
	
	FloatBuffer mTriangleBuffer;
	FloatBuffer mColorBuffer;
}
