package com.pgmriver.android;

import java.nio.FloatBuffer;

import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.opengl.GLU;

public class GLViewFourth extends BaseGLView {

	public GLViewFourth(Context context) {
		super(context, 20);
		// TODO Auto-generated constructor stub
		mSquareBuffer = makeFloatBuffer(square);
		mTriangleBuffer = makeFloatBuffer(triangle);
		mColorBuffer = makeFloatBuffer(colors);
	}

	@Override
	protected void init(GL10 gl)
	{
		gl.glClearColor(0, 0, 0, 0);
		
		gl.glMatrixMode(GL10.GL_PROJECTION);
		gl.glLoadIdentity();
		
		// 投影行列を作成し、それをカレントの投影行列に乗算する
		// クリッピング領域は左下(0.0f, 0.0f)、右上(1.3f, 1.0f)の矩形
		GLU.gluOrtho2D(gl, 0.0f, 1.3f, 0.0f, 1.0f);
		
		// GL_FLAT : 単色(全頂点同じ色)
		// GL_SMOOTH : 複数の色(全ての頂点が独立した色：頂点間の色が補間される)
		gl.glShadeModel(GL10.GL_SMOOTH);
	}
	@Override
	protected void drawFrame(GL10 gl) {
		// TODO Auto-generated method stub
		
		mXrot += 1f;
		mYrot += 1f;
		
		gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);
		
		gl.glMatrixMode(GL10.GL_MODELVIEW);
		gl.glLoadIdentity();
		
		gl.glPushMatrix();
		gl.glTranslatef(-0.25f, 0.0f, -2f);
		gl.glRotatef(mXrot, 1.0f, 0, 0);
		gl.glVertexPointer(3, GL10.GL_FLOAT, 0, mTriangleBuffer);
		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		gl.glColorPointer(4, GL10.GL_FLOAT, 0, mColorBuffer);
		gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
		gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, 0, 3);
		gl.glPopMatrix();
		
		gl.glDisableClientState(GL10.GL_COLOR_ARRAY);
		
		gl.glTranslatef(0.25f, 0.0f, -2f);
		gl.glRotatef(mYrot, 0, 1.0f, 0);
		gl.glVertexPointer(3, GL10.GL_FLOAT, 0, mSquareBuffer);
		gl.glColor4f(0.25f, 0.25f, 0.75f, 1.0f);
		gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, 0, 4);
	}

	float[] triangle = new float[] { 
			-0.25f, -0.25f, 0.0f,
			0.25f, -0.25f, 0.0f,
			-0.25f,  0.25f, 0.0f};

	float[] square = new float[] {
			-0.25f, -0.25f, 0.0f,
			0.25f, -0.25f, 0.0f,
			-0.25f, 0.25f, 0.0f,
			0.25f, 0.25f, 0.0f};
	
	float[] colors = new float[] {
			1, 0, 0, 1,
			0, 1, 0, 1,
			0, 0, 1, 1};
	
	FloatBuffer mSquareBuffer;
	FloatBuffer mTriangleBuffer;
	FloatBuffer mColorBuffer;
	
	float mXrot, mYrot;
}
